#ifndef _DISK_INCLUDED_
#define _DISK_INCLUDED_     /* definujeme, ze je inkludovan disk.h */
/*
 *  disk.h
 *  zde jsou funkce ktere pracuji s diskem
 *  funkce:
 *    resetdisk()       - resetuje floppy disk 0x00
 *    bios_read_disk()  - precte urcitou cast disku a nahraje ji do pameti
 *    readdisk()        - -//-, akorat vstup neni CHS, ale primo fyz. adresa disku
 */
  
#include <asm.h>
#include <types.h>
/*
 *  resetdisk()
 *  restartuje floppy A: 
 */
void resetdisk() {
resetd:
  asm mov ax, 0  ;
  asm mov dl, 0  ;
  asm int 13h    ;
  asm jc resetd  ;
}

/*
 *  bios_read_disk()
 *  nahraje do pameti <offset> <sectors> sektoru, na pozici CHS 
 */
void bios_read_disk(byte cylinder, byte head, byte sector, byte sectors, void *offset) {
  resetdisk();
  asm cli;
  _ES = _DS;             /* BUFF */
  _BX = (int)offset;     /* BUFF */  
  
  _AH = 2;               /* READ */
  _AL = sectors;         /* pocet sektoru */
  
  _CH = cylinder;        /* cislo stopy */
  _DH = head;            /* cislo hlavy */
  _CL = sector;          /* cislo sektoru */
  
  _DL = 0;               /* cislo drive */
  asm sti;
  __int__(0x13);

  return;
}

/*
 *  readdisk()
 *  nacte do <buffer> cast disku od <start> s poctem <sectors> sektoru
 *  - start je fyzicka adresa disku, prevadi se na CHS  
 */ 
void readdisk(int start, int sectors, void *buffer) {
  unsigned int c=0,
               h=0,
               s=start /*((addr/512)+1)*/ ;
  
  if(s > 36) {
    c = (s/36);
    s = (s%36);
  }
  
  if(s > 18) {
    h = (s/18);
    s = (s%18);
  }
      
  bios_read_disk((byte)c,(byte)h,(byte)s, sectors, buffer);
  return;
}

#endif
