#ifndef _FAT_INCLUDED_
#define _FAT_INCLUDED_
/*
 *  fat.h
 *  soubor pro praci s FAT12 a ruzne definovani typu pro nasi potrebu
 *  a jednoduchost :)
 *  typy:
 *    BootRecord      - MBR, rozdeleny na casti jako je pocet FAT, sektoru na klastr atp.
 *    FileTime        - format casu u souboru
 *    FileDate        - dormat data u souboru
 *    DirectoryEntry  - 32 bajtu pro asociaci s FAT + 12 bajtu pro realne jmeno
 *  funkce:
 *    getRealFileName() - z Name a Ext vytvori nazev souboru s "." 
 *    assocfile()       - asociuje file ve formatu Dir.Entry k urcite casti pameti
 *    findfile()        - hleda slozky a soubory
 */
  
#include <disk.h>
#include <global.h>
#include <malloc.h>
#include <printf.h>
#include <string.h>
#include <types.h>

/******************************************************************************/
/****************************   DEFINICE TYPU   *******************************/
/******************************************************************************/
/* atributy souboru */
#define DELETED (char)0xE5
#define PART_PRIMARY 0
#define PART_LOGICAL 1
#define PART_MBR     2
#define PART_FLOPPY  3
#define PART_SBM     4

#define F_READONLY  1
#define F_HIDDEN    2
#define F_SYSTEM    4
#define F_VOLUME    8
#define F_DIRECTORY 16
#define F_ARCHIVE   32

/* atributy klastru */
#define FAT_AVAILABLE    0x0000
#define FAT_INVALIDENTRY 0x0001
#define FAT_RESERVEDs    0x0ff0
#define FAT_RESERVEDe    0x0ff6
#define FAT_BADSECTOR    0x0ff7
#define FAT_LASTCLUSTERs 0x0ff8
#define FAT_LASTCLUSTERe 0x0fff

/* struktura MasterBootRecordu */
typedef struct {
	word	jump;
	byte	nop;
	byte	oem[8];
	word	sector_size;
	byte	cluster_size;
	word	sector_reserved;
	byte	fat_count;
	word	root_size;
	word	total_sectors16;
	byte	media;
	word	fat_size16;
	word	sectors_per_track;
	word	head_count;
	dword	sectors_hidden;
	dword	total_sectors32;
	byte	phys_drv;
	byte	reserved;
	byte	ext_sign;
	dword	serial;
	byte	label[11];
	byte	fs_id[8];
	byte	data[512-62-2];
	word	boot_sign;
	} BootRecord;

/* struktura "casu" u souboru */
typedef struct {
	word
		sec          :5,
		min          :6,
		hour         :5;
	} FileTime;

/* struktura "datumu" u souboru */
typedef struct {
  word
  	day          :5,    /* 5 je pocet bitu kolikami se vyjadruje pocet dni v mesici 2^5 = 32-1 = 31dni */
  	month        :4,    /* 2^4-1=15 max 15 mesicu, nam staci jen 12 :) */
  	year         :7;
  } FileDate;
  
/* struktura celeho souboru ve FAT */
typedef struct {
  byte name[8];
  byte ext[3];
  byte attr;            /* atribut */
  byte reserved[0x0A];
  FileTime time;
  FileDate date;
  word ClstrNo;
  dword filesize;
  byte realname[12];   /* skutecne jmeno + pripona */
  } DirectoryEntry;

/******************************************************************************/
/********************************   FUNKCE   **********************************/
/******************************************************************************/
/* globalni promenna <*mbr> */
BootRecord *mbr;

/*
 *  getRealFileName()
 *  vytvori nazev souboru s teckou a bez mezer z <name> a <ext>
 *  a vrati ho v <realname> 
 */  
char *getRealFileName(char *name, char *ext, char *realname) {
int cykl = 0;
   while((*name != ' ') && (*name != '\0') && (cykl != 8)) {        
    *realname = *name;
    realname++;
    name++;
    cykl++;
  }
  if(!(*(ext)==' ' && *(ext+1)==' ' && *(ext+2)==' ')) {
    *realname++ = '.';

    cykl = 0;  
    while((*ext != ' ') && (*ext != '\0') && (cykl != 3)) {    
      *realname = *ext;
      realname++;
      ext++;
      cykl++;
    }
  }
  *realname = '\0';
  return realname;  
}

/*
 *  assocfile()
 *  asociuje hodnoty pro <file>
 */  
void assocfile(char *mem, DirectoryEntry *file) {
  memcpy(file->name,         (mem), 8);
  memcpy(file->ext,          (mem+8), 3);
  file->attr     = *(byte*)  (mem+11);
  file->ClstrNo  = *(word*)  (mem+0x1A);
  file->filesize = *(dword*) (mem+0x1C);
  getRealFileName(file->name, file->ext, file->realname);
  return;
}

/*
 *  findfile()
 *  hleda soubor <filename>, vraci 0 pri nenalezenem,
 *  1 pri nalezenem souboru , a nastavi do <file> dane atributy
 */ 
int findfile(DirectoryEntry *file, int pos) {
signed int _ret   = pos;
int _stranka      = 0;
int _pos          = 0;
char *bkp_sdBuff;                           /* zaloha sdBuff (viz. nize) */
char *sdBuff      = (char*)malloc(0x200);   /* alokujeme 512 bajtu pro sektor */

bkp_sdBuff = sdBuff;

_stranka = (pos/16);          /* ((pos*32)/512) zjistime na kolikate 512ti bajtove strance je pointer na dalsi fajl */
_pos     = (pos%16);          /* (((pos*32)%512)*32) */

  readdisk((dir_ptr+_stranka), 1, sdBuff);
  sdBuff += (_pos*32);
  while(1) { 
  if (*sdBuff == '\0') { _ret = -1; break; }
  if ((*sdBuff == DELETED) || (*(sdBuff+0x0B) == 0x08)) { /* vymazany soubor? Label? Dekuji, nechci */
     sdBuff += 32;
     _ret++;
     _pos++;
     if(_pos >= 16) {
       _stranka = (_pos/16);
       _pos     = (_pos%16);
       readdisk((dir_ptr+_stranka), 1, sdBuff);
       sdBuff += (_pos*32);
     }
    } else {
      assocfile(sdBuff, file);
      sdBuff  += 32;
      _ret++;
      _pos++;
      break;
    }
  }
  free(bkp_sdBuff);                         /* zde dealokujeme puvodni sdBuff */
  return (_ret);
}

#endif
